% Matlab code required for computing the numerical examples in 4 of 
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [R] = Iwen_sparse(A,k,B,N)
%IWEN_SPARSE Find a k-sparse representation R of the B-sparse function f 
% with even bandwidth N in sublinear time. Only frequencies in 
% {-N/2+1,...,N/2} are considered. A has to be the matrix containing the 
% necessary samples f(2*pi*a/(p(l+1)*p_Q(j+1))) of f, where the first 
% p(l+1)*p_Q(j+1) entries of the l+1-th column and j+1-th layer of A are 
% the values f(2*pi*a/(p(l+1)*p_Q(j+1))) for a in 
% {0,...,p(l+1)*p_Q(j+1)-1}, l in {0,...,m} and j in {0,...,K}. Return the 
% matrix R containing the frequencies and coefficient estimates.

if (k<=B) && (B<=N)
    % Initialise the output vector R.
    R=zeros(k,2);
    %% Preliminary computations
    % Let p be the vector consisting of 1 and the m smallest primes with
    % p_(2)*...*p_(m+1)>=N/B.
    [m,p]=findm(B,N);
    % Find the smallest prime p_q greater than max(B(k),p(m+1)).
    p_q=qthprime(max(B,p(m+1)));
    % Compute the number K+1 of required S_j-primes.
    K=2*B*floor(log(N)/log(p_q));
    % Compute p_Q, the vector containing the S_j-primes primes 
    % p_q,...,p_{q+K}.
    p_Q=nprimes(K+1,p_q);
    
    %% Computation of the DFTs
    % Initialise DFTA, the 3-dimensional array containing the DFTs of 
    % length p(l+1)*p_Q(j+1) of the first p(l+1)*p_Q(j+1) entries of the 
    % l+1-th column and the j+1-th layer of A for l in {0,...,m} and j in 
    % {0,...,K}.
    DFTA=zeros(p(m+1)*p_Q(K+1),m+1,K+1);
    for j=0:K
        for l=0:m
            % Compute DFTA layer- and  columnwise, i.e. for fixed p(l+1) 
            % and p_Q(j+1).
            DFTA(1:p(l+1)*p_Q(j+1),l+1,j+1)=fft(A(1:p(l+1)*p_Q(j+1),l+1,j+1))*2*pi/(p(l+1)*p_Q(j+1));
        end
    end
    
    %% Identification of the Energetic Frequencies
    % Initialise nu, the matrix containing for each j in {0,...,K} the B 
    % most energetic frequencies of f for each j in {0,...,K}.
    nu=zeros(K+1,B);
    % For each j in {0,...,K} compute the B most energetic frequencies with
    % the CRT reconstruction procedure.
    for j=0:K
        % Initialise P, the vector containing the moduli p_Q(j+1) and 
        % p(2),...,p(m+1) for the CRT reconstruction.
        P=[p_Q(j+1) p(2:m+1)];        
        % Initialise r, the vector containing the residues modulo p_Q(j+1) 
        % and p(2),...,p(m+1) of the B most energetic frequencies.
        r=zeros(m+1,B);
        % Sort the DFT of length p_Q(j+1) by magnitude in descending order 
        % and store the previous indices of the elements in index.
        [~,index]=sort(abs(DFTA(1:p_Q(j+1),1,j+1)),'descend');
        for t=1:B
            % The residue modulo p_Q(j+1) of the frequency associated to b 
            % is the index of the b-th largest magnitude entry of the DFT 
            % of length p_Q(j+1) in the range {0,...,p_Q(j)-1}.
            r(1,t)=index(t)-1;
            % Find the residues r(l,b) modulo p(l+1) for l in {1,...,m}.
            for l=1:m
                % Compute C, the vector containing the absolute values of 
                % the differences between DFTA(r(1,b)+1,1,j+1) and 
                % DFTA(t*p_Q(j+1)+r(1,b)+1,l+1,j+1) for b in 
                % {0,...,p(l+1)-1}.
                C=zeros(p(l+1),1);
                for b=0:p(l+1)-1
                    C(b+1)=abs(DFTA(r(1,t)+1,1,j+1)-DFTA(b*p_Q(j+1)+r(1,t)+1,l+1,j+1));
                end
                % Find the correct value for b_min as the index of the minimal 
                % element of C.
                [~,b_min]=min(C);
                % Shift b_min into the range {0,...,p(l+1)-1}.
                b_min=b_min-1; 
                % Compute the residue r(l+1,t) of the frequency mod p(l+1).
                r(l+1,t)=mod(b_min*p_Q(j+1)+r(1,t),p(l+1));
            end
            
            %% Reconstruction of nu(j+1,b) from its residues r(1,b),...,r(m+1,b) modulo p_Q(j+1),p(2),...,p(m+1) 
            nu(j+1,t)=reconstruction(r(:,t),P,N);
        end
    end
    % Find out often the frequencies were reconstructed. nu_un contains the 
    % unique frequencies of nu and nu_nr is the number of times the
    % elements of nu_un were reconstructed.
    [nu_un,~,index]=unique(nu);
    nu_nr=accumarray(index,1);
    
    %% Estimation of the Coefficients
    % Initialise Q, the matrix containing the frequencies from nu_un in the 
    % first column and the coefficient estimates in the second column.
    Q=zeros(length(nu_un),2);
    Q(:,1)=nu_un;
    % Compute the coefficient estimates for the at most B frequencies that 
    % were reconstructed more than (K+1)/2 times.
    for j=1:length(nu_un)
        if nu_nr(j)>(K+1)/2
            % Initialise RE and IM, the vectors containing the K+1 real and 
            % imaginary parts of the (nu_un(j) modulo p(m+1)*p_Q(h+1))-th 
            % row, m+1-th column and j+1-th layer of DFTA for h in 
            % {0,...,K}.
            RE=zeros(K+1,1);
            IM=zeros(K+1,1);
            % Compute the real and imaginary parts.
            for h=0:K
                RE(h+1)=real(DFTA(mod(nu_un(j),p(m+1)*p_Q(h+1))+1,m+1,h+1));
                IM(h+1)=imag(DFTA(mod(nu_un(j),p(m+1)*p_Q(h+1))+1,m+1,h+1));
            end
            % Insert the coefficient estimate for nu, computed from the 
            % medians of the real and imaginary parts, into the second 
            % column of Q.
            Q(j,2)=median(RE)+1i*median(IM);
        end
    end
    
    %% Identification of the k most energetic frequencies
    % Sort the coefficient estimates in Q by magnitude.
    [Est_sort,index2]=sort(Q(:,2),'descend'); 
    % Sort the frequencies in Q in the same way as the estimates, using I, 
    % the vector containing the indices of the estimates in Q(:,2), 
    % so that the pairs of frequencies and coefficients are restored.
    nu_sort=Q(:,1);
    R_aux=[nu_sort(index2) Est_sort];     
    % For the output vector R take only the k elements of R_aux that have 
    % the largest magnitude.
    for j=1:k       
        R(j,:)=R_aux(j,:);
        R(j,2)=R(j,2)/(2*pi);
    end        
else
    disp('The sparsity  estimate B has to be at least k and at most N.')
end
end

