% Matlab code example for Algorithm 1 presented in
% "Sparse FFT for Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [R] = SFFT_short_support(A,B,N,epsilon)
% SFFT_SHORT_SUPPORT Find the energetic frequencies and estimates for their 
% Fourier coefficients of a function f with short support bounded by B and
% even bandwidth N. Only frequencies in {-N/2+1,...,N/2} are considered. A
% frequency is energetic if the absolute value of its Fourier coefficient 
% is greater than epsilon. A has to be the matrix containing the necessary 
% samples f(2*pi*m/(t(l)*s)) of f, where the first t(l)*s entries
% of the l-th column of A are the values f(2*pi*a/(t(l)*s)) for a in 
% {0,...,t(l)*s-1} and l in {1,...,L}. s is the smallest power of 2 that is
% greater than B, t(1)=1 and t(2),...,t(L) are the smallest odd primes such
% that t(2)*...*t(L)>=N/B. Return the matrix R containing the frequencies 
% and coefficient estimates.

if (B<=N) && (mod(N,2)==0)
    %% Preliminary Computations
    % Find the first L-1 odd primes such that their product is greater than
    % N/B.
    [L,t]= findL(B,N);
    % Set s as the smallest power of 2 that is greater than B.
    s=2^(floor(log(B)/log(2))+1);
    % Computation of the DFTs: Initialise DFTA, the matrix containing the 
    % DFTs of length t(l)*s of the first t(l)*s entries of the l-th column 
    % of A for l in {1,...,L}.
    DFTA=zeros(t(L)*s,L);
    % Compute the DFTs of the first t(l)*s entries of the l-th column of A.
    for l=1:L
        DFTA(1:t(l)*s,l)=fft(A(1:t(l)*s,l))*2*pi/(t(l)*s);
    end
    %% Identification of an Energetic Frequency nu
    % Initialise r, the vector containing the residues of nu modulo s and 
    % t(2),...,t(L) for the CRT reconstruction.
    r=zeros(L,1);
    % Find the index of the entry of the DFT of length s that has the
    % largest absolute value. Then there is an energetic frequency nu whose
    % residue modulo s is this index. 
    [~,index]=max(abs(DFTA(:,1)));
    % Shift the residue of nu modulo s into the range {0,...,s-1}.
    r(1)=index-1;
    % Find the residues r(l) modulo t(l) of nu for l in {2,...,L}.
    for l=2:L
        % Compute C, the vector containing the absolute values of the 
        % differences between DFTA(r(1)+1,1) and DFTA(b*s+r(1)+1,l) for 
        % b in {0,...,t(l)-1}.
        C=zeros(t(l),1);
        for b=0:t(l)-1
            C(b+1)=abs(DFTA(r(1)+1,1)-DFTA(b*s+r(1)+1,l));
        end
        % Find the correct value for b_min as the index of the minimal 
        % element of C.
        [~,b_min]=min(C);
        % Shift b_min into the range {0,...,t(l)-1}.
        b_min=b_min-1; 
        % Compute the residue r(l) of nu mod t(l).
        r(l)=mod(b_min*s+r(1),t(l));
    end
    % Reconstruct nu from its residues r(1),...,r(L) modulo s,t(2),..,t(L).
    nu=reconstruction(r,[s t(2:L)],N);
    
    %% Identification of the remaining frequencies and coefficients  
    % Initialise R, the matrix containing the energetic frequencies in
    % the first column and the coefficient estimates in the second column. 
    M=min(nu+B-1,N/2)-max(nu-B+1,-N/2+1)+1;
    % Compute M, the number of frequencies in {nu-B+1,...,nu+B-1} that are 
    % contained the allowed interval {-N/2+1,...,N/2}.
    R=zeros(M,2);
    % Check whether the frequencies in {nu-B+1,...,nu+B-1} that are also in 
    % {-N/2+1,...,N/2} are energetic.
    for j=max(nu-B+1,-N/2+1):min(nu+B-1,N/2)
        % R(:,1) will contain all 2*B-1 possible frequencies in order to be 
        % able to embed R into an N-length vector.
        R(j-max(nu-B+1,-N/2+1)+1,1)=j;
        if abs(DFTA(mod(j,t(2)*s)+1,2))>epsilon
            % If an energetic frequency is found, add its Fourier
            % coefficient to R(j,2). Since 2*B-1 distinct frequencies
            % are considered, one needs to take the Fourier coefficient 
            % from the DFT of length t(2)*B=3*B.
            R(j-max(nu-B+1,-N/2+1)+1,2)=1/(2*pi)*DFTA(mod(j,t(2)*s)+1,2);
        end
    end
else
    disp('Support length cannot be greater than the bandwidth and N has to be even.')
end
end

