% Matlab code for computing the numerical examples (runtimes and 
% reconstruction errors) in 4 of "Sparse FFT for
% Functions with Short Frequency Support" by Sina Bittens

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = TEST_SFFT_short_support(tests,B,N,epsilon,variant)
% TEST_SFFT_SHORT_SUPPORT Computes average runtime and reconstruction error 
% of SFFT_short_support, Iwen's sparse FFT algorithm (Iwen_sparse) and 
% Matlab's fft over tests tries for a function with short support bounded 
% by B and even bandwidth N. 
% 
% If variant=0, the average runtimes of SFFT_short_support and fft are 
% computed for noiseless data. If variant=1, we compute the average 
% runtimes of ifft and Iwen_sparse for noiseless data. If variant=2, we 
% compute the average reconstruction errors for noisy signals (uniform 
% noise with SNR values [0 5 ... 50]) for SFFT_short_support and fft.

%% Preliminary Computations
if (sum(B<=N)==length(B)) && (mod(N,2)==0)
    format long
    b=length(B);
    % For all b support lengths create tests random real and imaginary 
    % parts with absolute values in [0,10] for the B(k) Fourier 
    % coefficients. 
    Re=zeros(tests,max(B),b);
    Im=zeros(tests,max(B),b);
    for k=1:b
        Re(:,1:B(k),k)=rand(tests,B(k))*20-10;
        Im(:,1:B(k),k)=rand(tests,B(k))*20-10;
    end
    % Create the coefficient matrix.
    Coeff=Re+1i*Im;
    % For all b support lengths and all tests functions create a random 
    % start frequency nu such that nu,nu+1,...,nu+B(l)-1 are contained in 
    % {-N/2+1,...,N/2}. 
    nu=zeros(tests,b);
    for k=1:b
        nu(:,k)=randi([-N/2+1 N/2-B(k)+1],tests,1);
    end
    % Initialize matrix for the average runtimes.
    time=zeros(b,5);

    % Run the three methods for every support length B(k) and for all tests 
    % functions.
    for k=1:b
        % Initialize runtime counters.
        time_SFFT_shortsupp=0;
        time_fft=0;
        time_Iwen_sparse=0;
        
        % Define the SNR values for noisy signals.
        SNR=zeros(11,1);
        for l=1:length(SNR)
            SNR(l)=(l-1)*5;
        end
        % Initialize array for the average noisy reconstruction errors.
        error=zeros(length(SNR),3,b);
        error(:,1,k)=SNR;
        
        %% Preliminary Computations for SFFT_short_support.
        % Find the first L-1 odd primes such that their product is 
        % greater than N/B(k).
        [L,t]= findL(B(k),N);
        % Set s as the smallest power of 2 that is greater than B.
        s=2^(floor(log(B(k))/log(2))+1);
        
        %% Preliminary Computations for Iwen_sparse
        % Let p be the vector consisting of 1 and the m smallest 
        % primes with p(2)*...*p(m+1)>=N/B.
        [m,p]=findm(B(k),N);
        % Find the smallest prime p_q greater than
        % max(B(k),p(m+1)).
        p_q=qthprime(max(B(k),p(m+1)));
        % Compute the number K+1 of required S_j-primes.
        K=2*B(k)*floor(log(N)/log(p_q));
        % Compute p_Q, the vector containing the S_j-primes 
        % p_q,...,p_{q+K}.
        p_Q=nprimes(K+1,p_q);
        
        for j=1:tests        
            %% Standard FFT Algorithm
            % Compute A_fft, the vector containing the necessary samples 
            % f(2*pi*a/N) of f for a in {0,...,N-1}. Additionally, save
            % A_fft_hat, the N-length vector consisting of the Fourier 
            % coefficients given by Coeff(j,:,l) at the positions 
            % corresponding to the respective frequencies.
            [A_fft,A_fft_hat]=exact_eval_N(N,B(k),nu(j,k),Coeff(j,:,k));
            % Measure the runtime of fft.
            tic;
            S_fft=fft(A_fft);
            time_fft=time_fft+toc;

            %% SFFT for Short Support
            if variant ~=1
                % Computation of the DFTs: Initialise A_SFFT_shortsupp, the 
                % matrix containing estimates of the necessary samples 
                % f(2*pi*a/(t(l)*s)) of f, where the first t(l)*s entries 
                % of the l-th column of A_SFFT_shortsupp are the estimates 
                % for f(2*pi*a/(t(l)*s)) for a in {0,...,t(l)*s-1} and l in 
                % {1,...,L}.
                A_SFFT_shortsupp=zeros(t(L)*s,L);
                % Compute A_SFFT_shortsupp columnwise.
                for l=1:L
                    A_SFFT_shortsupp(1:t(l)*s,l)=exact_eval(t(l)*s,B(k),nu(j,k),Coeff(j,:,k));
                end
                % Measure the runtime of SFFT_short_support.
                tic;
                R_SFFT_shortsupp=SFFT_short_support(A_SFFT_shortsupp,B(k),N,epsilon);
                time_SFFT_shortsupp=time_SFFT_shortsupp+toc;
            end

            %% Iwen's Sparse FFT Algorithm
            if variant ==1
                % Initialise A_Iwen_sparse, the 3-dimensional array 
                % containing the necessary samples 
                % f(2*pi*a/(p(l+1)*p_Q(z+1))) of f, where the first 
                % p(l+1)*p_Q(z+1) entries of the l+1-th column and z+1-th 
                % layer of A are the values f(2*pi*a/(p(l+1)*p_Q(z+1))) for 
                % a in {0,...,p(l+1)*p_Q(z+1)-1}, l in {0,...,m} and z in 
                % {0,...,K}.
                A_Iwen_sparse=zeros(p(m+1)*p_Q(K+1),m+1,K+1);
                % Compute A_Iwen_sparse layer- and columnwise, i.e., for 
                % fixed p(l+1) and p_Q(z+1).
                for z=0:K
                    for l=0:m
                        A_Iwen_sparse(1:p(l+1)*p_Q(z+1),l+1,z+1)=exact_eval(p(l+1)*p_Q(z+1),B(k),nu(j,k),Coeff(j,:,k));
                    end
                end
                % Measure the runtime of Iwen_sparse, where the 
                % reconstruction is done with sparsity B(k) and sparsity 
                % estimate B(k).
                tic;
                R_Iwen_sparse=Iwen_sparse(A_Iwen_sparse,B(k),B(k),N);
                time_Iwen_sparse=time_Iwen_sparse+toc;
            end

            %% Average Reconstruction Error for Noisy Input Data
            if variant==2
                for n=1:length(SNR)
                    % Create random uniformly distributed noise for the 
                    % coefficient vector, which has to be of length N, 
                    % because frequencies outside of the support can have 
                    % non-zero coefficients.
                    noise=rand(N,1)-0.5+1i*(rand(N,1)-0.5);
                    % Scale the noise according to the momentary SNR value
                    % SNR(n).
                    noise=norm(Coeff(j,1:B(k),k))*10^(-SNR(n)/20)/norm(noise)*noise;
                    % Initialize embedding vector.
                    Coeff_noise=zeros(N,1);
                    % Insert the original coefficients at the positions in 
                    % Coeff_noise corresponding to the respective 
                    % frequencies.
                    Coeff_noise(nu(j,k)+N/2-1+1:nu(j,k)+N/2-1+B(k)-1+1)=Coeff(j,1:B(k),k); 
                    % Create noisy input data by addition.
                    Coeff_noise=Coeff_noise+noise;

                    %% SFFT_short_support for Noisy Input Data 
                    % Computation of the DFTs: Initialise 
                    % A_SFFT_shortsupp_noise, the matrix containing the 
                    % samples f(2*pi*a/(t(l)*s)), computed from the noisy 
                    % coefficients.
                    A_SFFT_shortsupp_noise=zeros(t(L)*s,L);
                    % Compute A_SFFT_shortsupp_noise columnwise. 
                    for l=1:L
                        A_SFFT_shortsupp_noise(1:t(l)*s,l)=noisy_eval(t(l)*s,N,Coeff_noise);
                    end
                    % Apply SFFT_short_support to the noisy input data.
                    R_SFFT_shortsupp_noise=SFFT_short_support(A_SFFT_shortsupp_noise,B(k),N,epsilon);
                    % Insert the frequencies and their coefficient 
                    % estimates given by R_SFFT_shortsupp_noise correctly 
                    % into an Nx2 matrix to be able to compare them to the 
                    % result given by fft for a vector of length N.
                    % Identify the first, not necessarily energetic 
                    % frequency in R_SFFT_shortsupp_noise.
                    omega_noise=R_SFFT_shortsupp_noise(1,1);
                    % Compute the number of frequencies in
                    % R_SFFT_shortsupp_noise.
                    nrnoise=size(R_SFFT_shortsupp_noise);
                    % Initialize the embedding vector 
                    % S_SFFT_shortsupp_noise.
                    S_SFFT_shortsupp_noise=zeros(N,1);
                    % Insert the coefficient estimates from 
                    % R_SFFT_shortsupp_noise at the positions in 
                    % S_SFFT_shortsupp_noise corresponding to the 
                    % respective frequencies.
                    S_SFFT_shortsupp_noise(omega_noise+N/2-1+1:omega_noise+N/2-1+nrnoise(1)-1+1)=R_SFFT_shortsupp_noise(:,2);

                    %% Standard FFT Algorithm for Noisy Input Data
                    % Compute A_fft_noise, the vector containing the 
                    % samples f(2*pi*a/N) for noisy data. 
                    A_fft_noise=ifft(Coeff_noise);
                    % Apply fft to the noisy data.
                    S_fft_noise=fft(A_fft_noise);

                    % Compute the reconstrucion errors between the 
                    % original, noiseless data A_fft_hat and the outputs of 
                    % SFFT_short_support and fft, respectively.
                    error(n,2,k)=error(n,2,k)+norm(S_SFFT_shortsupp_noise-A_fft_hat)/N;
                    error(n,3,k)=error(n,3,k)+norm(S_fft_noise-A_fft_hat)/N;  
                end
            end
        end
        % Compute the average runtimes and the average reconstruction 
        % errors.
        time(k,1)=B(k);
        time(k,2)=time_SFFT_shortsupp/tests;
        time(k,3)=time_fft/tests;
        time(k,4)=N;
        time(k,5)=time_Iwen_sparse/tests;
        error(:,2:3,:)=error(:,2:3,:)/tests;    
        % Save the matrix of reconstruction error values in a separate file
        % for each support length B(k).
        if variant ==2
            filename = ['errorSFFTshortsupp' num2str(B(k)) '.dat'];
            dlmwrite(filename, error(:,:,k), 'delimiter', '\t');
        end
    end
    if variant ==0
        % Save the matrix of runtime values in a file.
        dlmwrite('runtimeSFFTshortsupp.dat',time(:,1:4),'delimiter','\t');
    end
    if variant ==1
        % Save the matrix of runtime values for Iwen's sparse algorithm 
        % in a separate file.
        dlmwrite('runtimeIwensparse.dat',time(:,5),'delimiter','\t');
    end
else
    disp('Support length cannot be greater than the bandwidth and N has to be even.')
end
end