% Matlab code for computing the numerical examples (sampling requirements) 
% in 4 of "Sparse FFT for Functions with Short Frequency Support" by 
% Sina Bittens
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [] = required_samples(B,N)
% REQUIRED_SAMPLES Compute the ratio between the number of samples 
% necessary to execute SFFT_short_support, Iwen's sparse FFT algorithm 
% (Iwen_sparse) and the sparse FFT algorithm for functions with short 
% support by Plonka and Wannenwetsch, and the bandwidth N=2^J for 
% functions/vectors with short support bounded by B.
if (max(B)<=min(N)) && (sum((abs(round(log(N)/log(2))-log(N)/log(2))) <= eps('double'))==length(N))
    % Initialise matrices containing the sampling requirements for the
    % different values of B and N and the values of B and N.
    Samples_SFFT_shortsupp=zeros(length(N)+1,length(B)+1);
    Samples_Iwen=zeros(length(N)+1,length(B)+1);
    Samples_Plonka_Wann=zeros(length(N)+1,length(B)+1);
    Samples_SFFT_shortsupp(2:length(N)+1,1)=N;
    Samples_Iwen(2:length(N)+1,1)=N;
    Samples_Plonka_Wann(2:length(N)+1,1)=N;
    Samples_SFFT_shortsupp(1,2:length(B)+1)=B';
    Samples_Iwen(1,2:length(B)+1)=B';
    Samples_Plonka_Wann(1,2:length(B)+1)=B';
    for n=1:length(N)
       for b=1:length(B)
            %% Sampling Requirements for SFFT_short_support
            % Find the first L-1 odd primes such that their product is 
            % greater than N/B.
            [~,p]=findL(B(b),N(n));
            % Set s as the smallest power of 2 that is greater than B.
            s=2^(floor(log(B(b))/log(2))+1);
            % Compute the ratio between the number of samples and the 
            % bandwidth N(n).
            Samples_SFFT_shortsupp(n+1,b+1)=s*sum(p)/N(n);
            
            %% Sampling Requirements for Iwen's Sparse FFT Algorithm
            % Let p_sparse be the vector consisting of 1 and the m 
            % smallest primes with p_sparse(2)*...*p_sparse(m+1)>=N/B.
            [m,p_sparse]=findm(B(b),N(n));
            % Find the smallest prime p_q greater than
            % max(B(k),p_sparse(m+1)).
            p_q=qthprime(max(B(b),p_sparse(m+1)));
            % Compute the number K_sparse+1 of required S_j-primes.
            K=2*B(b)*floor(log(N(n))/log(p_q));
            % Compute p_Q, the vector containing the S_j-primes 
            % p_q,...,p_{q+K_sparse}.
            p_Q=nprimes(K+1,p_q);  
            % Compute the number of required samples.
            a=0;
            for j=1:K+1
                for l=1:m+1
                    a=a+p_sparse(l)*p_Q(j);
                end
            end
            % Divide the number of required samples by the bandwidth N(n).
            Samples_Iwen(n+1,b+1)=a/N(n);

            %% Sampling Requirements for the SFFT Algorithm by Plonka and Wannenwetsch
            J=log(N(n))/log(2);
            L=ceil(log(B(b))/log(2));
            % In most cases the first support index can be found by
            % considering one additional vector in step 2, so in total 
            % two DFTs of length 2^(L+1) have to be computed, and J-L-1
            % further samples are being used.
            it_nr=1;
            Samples_Plonka_Wann(n+1,b+1)=((1+it_nr)*2^(L+1)+J-L-1)/N(n);
       end
    end
    % Output
    dlmwrite('Samples_SFFT_shortsupp.dat',Samples_SFFT_shortsupp','delimiter','\t')
    dlmwrite('Samples_Iwen.dat',Samples_Iwen','delimiter','\t')
    dlmwrite('Samples_Plonka_Wann.dat',Samples_Plonka_Wann','delimiter','\t')
else
    disp('The sparsity B has to be less than N and N has to be a power of 2.')
end
end

